module One
  module Restore
    class HapyVM
        attr_reader :id, :info, :name, :disks, :images, :labels
        def initialize(vm, images)
            if images
                @images = self._update_image_pool(images)
            else
                @images = images
            end
            @info = vm.to_hash
            @id = @info["VM"]["ID"]
            @name = @info["VM"]["NAME"]
            @vm = vm
        end
    
        def _wait_for_it(st)
            cnt = 0
            sleep(1)
            while self.status != st
                cnt += 1
                sleep(1)
                if cnt == 120
                    break
                end
            end
        end
    
        def state
            @vm.info
            return @info["VM"]["STATE"]
        end
    
        def status
            @vm.info
            return @vm.status
        end
    
        def poweroff(hard=false)
            @vm.info
            @vm.poweroff(hard=true)
            self._wait_for_it("poff")
        end
    
        def resume
            @vm.info
            @vm.resume
            self._wait_for_it("runn")
        end


        def _getDest(filename)
            pth=Pathname.new(File.expand_path(filename))
            rlv=pth.relative_path_from(BACKUP_ROOT)
            return "#{ONEROOT}/#{rlv}"
        end

        def restore(path)
          vm = @vm
          id = @vm.id
          bck_dir = File.dirname(path)
          puts("Restoring vm #{id} from #{bck_dir}")
          vm.info
          vm.poweroff
          files = Find.find(bck_dir)
          files.each do |file|
              if File.directory?(file)
                  next
              end
              if File.symlink?(file)
                  # Follow the link
                  lkDest = File.readlink(file)
                  # Get the file shortname
                  lkDestFile = File.basename(lkDest)
      
                  # Find the backuped file
                  bckFile = ""
                  patt = ".*#{lkDestFile}"
                  Find.find(BACKUP_ROOT) do |path|
                      if path =~ /#{patt}/
                          bckFile = path
                      end
                  end
      
                  # Set the destination file for recover
                  dest=self._getDest(bckFile)
                  #Recover the file
                  sum = Digest::SHA1.file(bckFile)
                  if not One::Utils.copyImage(bckFile, dest, sum, compress=false)
                      LOGGER.error("VM #{id} #{vm.name} : Recover of #{bckFile} to #{dest} failed !")
                      puts("ERROR on #{bckFile} recovery from VM #{id}")
                  end
      
                  begin
                      ldest=self._getDest(file)
                      if File.symlink?(ldest)
                          if File.readlink(file) == File.readlink(ldest)
                              next
                          else
                              FileUtils.remove_entry(ldest)
                              FileUtils.copy_entry(file,ldest)
                          end
                      else
                          LOGGER.error("Source file [#{file}] is a symlink but destination [#{ldest}] is not")
                          puts("ERROR - Source file [#{file}] is a symlink but destination [#{ldest}] is not")
                      end
                  rescue Exception => e
                      LOGGER.error(e.message)
                      puts(e.message)
                  end
              else
                  # Set the destination file for recover
                  dest=self._getDest(file)
                  #Recover the file
                  sum = Digest::SHA1.file(file)
                  if not One::Utils.copyImage(file, dest, sum, compress=false)
                      LOGGER.error("VM #{id} #{vm.name} : Recover of #{file} to #{dest} failed !")
                      puts("ERROR on #{file} recovery from VM #{id}")
                  end
              end
          end
          vm.resume
      end
    end
  end
end